using System;
using System.IO;
using System.Net;
using System.Runtime.Serialization.Formatters.Binary;

namespace TourDataManager {
    public class CookieStorage {
        private readonly string cookiePath;
        
        private CookieContainer CookieContainer;

        public CookieStorage(string cookiePath){
            this.cookiePath = cookiePath;
        }

        public CookieContainer Get(){
            if (CookieContainer != null) return CookieContainer;
            CookieContainer = ReadCookiesFromDisk(cookiePath);
            Debug.Log($"Count of cookies restored : {CookieContainer.Count}");
            return CookieContainer;
        }

        public void Save(CookieContainer cookieContainer){
            WriteCookiesToDisk(cookiePath, cookieContainer);
            Debug.Log($"Count of cookies saved : {cookieContainer}");
        }
        
        public void Save(){
            Save(CookieContainer);
        }

        public static void WriteCookiesToDisk(string file, CookieContainer cookieJar){
            using(Stream stream = File.Create(file)){
                try {
                    Debug.Log("Writing cookies to disk... ");
                    var formatter = new BinaryFormatter();
                    formatter.Serialize(stream, cookieJar);
                    Debug.Log("Done.");
                } catch(Exception e) { 
                    Debug.LogError("Problem writing cookies to disk: " + e.GetType()); 
                }
            }
        }   

        public static CookieContainer ReadCookiesFromDisk(string file){

            try {
                using(Stream stream = File.Open(file, FileMode.Open)){
                    Debug.Log("Reading cookies from disk... ");
                    var formatter = new BinaryFormatter();
                    Debug.Log("Done.");
                    return (CookieContainer)formatter.Deserialize(stream);
                }
            } catch(Exception e) { 
                Debug.LogError("Problem reading cookies from disk: " + e.GetType()); 
                return new CookieContainer(); 
            }
        }
    }
}