using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Common;
using System.Data.Entity;
using System.Data.Entity.Core.Common;
using System.Data.Entity.Migrations;
using System.Data.SQLite;
using System.Data.SQLite.EF6;
using System.Linq;
using System.Threading.Tasks;
using SQLite.CodeFirst;
using TourDataManager.Entities;
// ReSharper disable UnusedAutoPropertyAccessor.Global

namespace TourDataManager {
    
    // ReSharper disable once InconsistentNaming
    public class SQLiteConfiguration : DbConfiguration{
        public SQLiteConfiguration(){
            SetProviderFactory("System.Data.SQLite", SQLiteFactory.Instance);
            SetProviderFactory("System.Data.SQLite.EF6", SQLiteProviderFactory.Instance);
            SetProviderServices("System.Data.SQLite", (DbProviderServices)SQLiteProviderFactory.Instance.GetService(typeof(DbProviderServices)));
        }
    }

    public class MyDbContext : DbContext{

        public MyDbContext(DbConnection connection, bool contextOwnsConnection) 
            : base(connection,contextOwnsConnection){
            
                DbConfiguration.SetConfiguration(new SQLiteConfiguration());
                Configuration.ProxyCreationEnabled = true;
                Configuration.LazyLoadingEnabled = true;
        }
  
        protected override void OnModelCreating(DbModelBuilder modelBuilder){

            // Эта штука отключает самостоятельную генерацию Id и позволяет мне самому устанавливать Id
            modelBuilder.Entity<Estate>().Property(estate => estate.Id)
                .HasDatabaseGeneratedOption(DatabaseGeneratedOption.None);
            // Ещё, вероятно, помогло бы [DatabaseGenerated(DatabaseGeneratedOption.None)]
            
            var sqliteConnectionInitializer = new SqliteDropCreateDatabaseWhenModelChanges<MyDbContext>(modelBuilder);
            Database.SetInitializer(sqliteConnectionInitializer);
        }

        public DbSet<Estate> Estates{ get; set; }
        
        


    }
    
    public class Db : IDisposable {
        private SQLiteConnection connection;
        private MyDbContext context;
        
        public Db(){
            connection = new SQLiteConnection("data source=mylovelybase.sqlite");
            context = new MyDbContext(connection,true);
        }

        public Estate[] GetEstates(){
            return context.Estates.ToArray();
        }
        public Task<Estate[]> GetEstatesAsync(){
            return context.Estates.ToArrayAsync();
        }

        public void InsertEstates(Estate[] estates){
            foreach (var estate in estates){
                context.Estates.AddOrUpdate(estate);
            }
            foreach (var contextEstate in context.Estates){
                Debug.Log(contextEstate.Id);
            }
            context.SaveChanges();
            
        }

        public void Dispose(){
            connection?.Dispose();
        }
    }
}