using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Ninject;
using TourDataManager.Entities;

namespace TourDataManager {
    public class FetchEstatesUseCase {
        
        
        [Inject] public HttpClient HttpClient{ get; set; }
        [Inject] public Db Database{ get; set; }
        
        public Task<Estate[]> FetchEstatesFromDbAsync(){
            return Database.GetEstatesAsync();
        }
        
        /// <summary>
        /// <exception cref="HttpRequestException">Если сервер даст дубу</exception>
        /// <exception cref="BigantoErrorException">Если вместо массива эстейтов с сервера получены ошибки</exception>
        /// <exception cref="JsonException">Если вместо массива эстейтов или ошибки пришло нечто иное</exception>
        /// </summary>
        public async Task<Estate[]> FetchEstatesFromServerAsync(){

            var response = await HttpClient.GetAsync("https://biganto.com/api-novus/estates.getList?client=desktopplayer&client_version=3.0&v=2.0");             
            // throw HttpRequestException
            var content = await response.Content.ReadAsStringAsync();

            ErrorContext error = null;
            var errorHandlerSettings = new JsonSerializerSettings{
                Error = (sender,args) => {
                    error = args.ErrorContext;
                    args.ErrorContext.Handled = true;
                }
            };
            
            var estates = JsonConvert.DeserializeObject<Estate[]>(content,errorHandlerSettings);
            if (error == null) return estates;
            error = null;
            
            // Вместо массива эстейтов в ответе может содержаться json ошибки
            // Предпринимается попытка десериализовать ошибки и выкинуть BigantoErrorException
            
            var exceptionList = JsonConvert.DeserializeObject<BigantoErrorList>(content,errorHandlerSettings);
            if (error == null) throw new BigantoErrorException(exceptionList);
            throw new JsonException("Something goes wrong",error.Error);
        }
    }
}