using System;
using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Ninject;
using TourDataManager.Entities;

namespace TourDataManager {
    public class FetchTourPreviewsUseCase {
        
        [Inject] public HttpClient HttpClient{ get; set; }
        
        public async Task<Tour[]> FetchTourFromServerAsync(long estateId){
            var response = await HttpClient.GetAsync(
                $"https://biganto.com/api-novus/tours.getList?client=desktopplayer&client_version=3.0&v=2.0&estate_id={estateId}");             
            // throw HttpRequestException
            var content = await response.Content.ReadAsStringAsync();
            
            // TODO Это в буквальнос смысле копипаста с FetchEstatesUseCase.FetchEstatesFromServerAsync
            ErrorContext error = null;
            var errorHandlerSettings = new JsonSerializerSettings{
                Error = (sender,args) => {
                    error = args.ErrorContext;
                    args.ErrorContext.Handled = true;
                }
            };
            
            var tours = JsonConvert.DeserializeObject<Tour[]>(content,errorHandlerSettings);
            if (error == null){
                // Т.к из json нам не приходит estateId, а мы и так по контексту его знаем, то вбиваем так.
                // Теперь и база будет знать.
                foreach (var tour in tours){tour.EstateId = estateId;}
                return tours;
            }
            error = null;
            
            
            var exceptionList = JsonConvert.DeserializeObject<BigantoErrorList>(content,errorHandlerSettings);
            if (error == null) throw new BigantoErrorException(exceptionList);
            throw new JsonException("Something goes wrong",error.Error);
        }
    }
}